﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Reflection;
using System.Resources;

#if AUTOCAD
using Autodesk.AutoCAD.AcInfoCenterConn;
using Autodesk.Internal.InfoCenter;
using AcadApp = Autodesk.AutoCAD.ApplicationServices;
using Autodesk.AutoCAD.DatabaseServices;

using AcRx = Autodesk.AutoCAD.Runtime;
using AcRuntime = Autodesk.AutoCAD.Runtime;
using AcApp = Autodesk.AutoCAD.ApplicationServices;
using AcDb = Autodesk.AutoCAD.DatabaseServices;
using AcGeometry = Autodesk.AutoCAD.Geometry;
using AcEditor = Autodesk.AutoCAD.EditorInput;
using AcGui = Autodesk.AutoCAD.GraphicsInterface;
using AcColors = Autodesk.AutoCAD.Colors;
using AcColor = Autodesk.AutoCAD.Colors.Color;
using AcWnd = Autodesk.AutoCAD.Windows;
#endif

#if BRICSCAD
using AcRx = Bricscad.Runtime;
using AcRuntime = Teigha.Runtime;
using AcApp = Bricscad.ApplicationServices;
using AcDb = Teigha.DatabaseServices;
using AcEditor = Bricscad.EditorInput;
using AcColors = Teigha.Colors;
using AcColor = Teigha.Colors.Color;
using AcWindows = Bricscad.Windows;
using AcGui = Teigha.GraphicsInterface;
using AcGeometry = Teigha.Geometry;
#endif

#if ZWCAD
using AcRx = ZwSoft.ZwCAD.Runtime;
using AcRuntime = ZwSoft.ZwCAD.Runtime;
using AcApp = ZwSoft.ZwCAD.ApplicationServices;
using AcDb = ZwSoft.ZwCAD.DatabaseServices;
using AcEditor = ZwSoft.ZwCAD.EditorInput;
using AcColors = ZwSoft.ZwCAD.Colors;
using AcColor = ZwSoft.ZwCAD.Colors.Color;
using AcWindows = ZwSoft.ZwCAD.Windows;
using AcGui = ZwSoft.ZwCAD.GraphicsInterface;
using AcGeometry = ZwSoft.ZwCAD.Geometry;
using ZwSoft.ZwCAD.DatabaseServices;
#endif

namespace Export3dSolids.AssemblyInfo
{
    internal partial class AssemblyInfo: Export3dSolids.ToolMain.mInitialize
    {
        /// <summary>
        /// Ermittelt die Informationen, die für die Assembly vorliegen
        /// (Name, Location(Pfad), DateiName(ohneExt), Befehle)
        /// </summary>
        public AssemblyInfo()
        {
            // Get the assembly, its name and location            
            Assembly assem = Assembly.GetExecutingAssembly();
            ApplicationName = assem.GetName().Name;
            String _Fullname = assem.Location;
            FileInfo _af = new FileInfo(_Fullname);
            ApplicationLocation = _af.DirectoryName;
            if (!ApplicationLocation.EndsWith("\\"))
                ApplicationLocation += "\\";

            ApplicationFilenBaseName = _af.Name.Substring(0, _af.Name.Length - _af.Extension.Length);

            //ApplicationLocation = assem.Location;

            // We'll collect information on the commands
            // (we could have used a map or a more complex
            // container for the global and localized names
            // - the assumption is we will have an equal
            // number of each with possibly fewer groups) 
            CommandsGlobal = new List<string>();
            CommandsLocal = new List<string>();
            CommandGroups = new List<string>();

            // Iterate through the modules in the assembly
            Module[] mods = assem.GetModules(true);
            foreach (Module mod in mods)
            {// Within each module, iterate through the types 
                Type[] types = mod.GetTypes();
                foreach (Type type in types)
                {
                    // We may need to get a type's resources
                    ResourceManager rm = new ResourceManager(type.FullName, assem);
                    rm.IgnoreCase = true;
                    // Get each method on a type
                    MethodInfo[] meths = type.GetMethods();
                    foreach (MethodInfo meth in meths)
                    {// Get the methods custom command attribute(s)
                        object[] attbs = meth.GetCustomAttributes(typeof(AcRuntime.CommandMethodAttribute), true);
                        foreach (object attb in attbs)
                        {
                            AcRuntime.CommandMethodAttribute cma = attb as AcRuntime.CommandMethodAttribute;
                            if (cma != null)
                            {
                                // And we can finally harvest the information
                                // about each command                                
                                string globName = cma.GlobalName;
                                string locName = globName;
                                string lid = cma.LocalizedNameId;

                                // If we have a localized command ID,
                                // let's look it up in our resources
                                if (lid != null)
                                {
                                    // Let's put a try-catch block around this
                                    // Failure just means we use the global
                                    // name twice (the default)
                                    try { locName = rm.GetString(lid); }
                                    catch { }
                                }
                                // Add the information to our data structures 
                                if (!CommandsGlobal.Contains(globName))
                                    CommandsGlobal.Add(globName);
                                if (!CommandsLocal.Contains(locName))
                                    CommandsLocal.Add(locName);
                                if (cma.GroupName != null && !CommandGroups.Contains(cma.GroupName))
                                    CommandGroups.Add(cma.GroupName);
                            }
                        }
                    }
                }
            }
        }
    }
}
