﻿using Microsoft.Win32;
using System;
using System.Collections.Generic;
using AcDb = Autodesk.AutoCAD.DatabaseServices;


namespace Export3dSolids
{

    internal partial class RegistryUpdate : Export3dSolids.ToolMain.mInitialize
    {
        public static void UnregisterForDemandLoading()
        { RemoveDemandLoadingEntries(true); }

        public static void CreateDemandLoadingEntries(string name, string path, List<string> globCmds, List<string> locCmds, List<string> groups, int flags, bool currentUser)
        {
            // Choose a Registry hive based on the function input
            RegistryKey hive = (currentUser ? Microsoft.Win32.Registry.CurrentUser : Microsoft.Win32.Registry.LocalMachine);
            // Open the main AutoCAD (or vertical) and "Applications" keys
            String ackString = "";
            if (AcDb.HostApplicationServices.Current.GetType().GetProperty("RegistryProductRootKey") != null)
                ackString = AcDb.HostApplicationServices.Current.GetType().GetProperty("RegistryProductRootKey").GetValue(AcDb.HostApplicationServices.Current, null).ToString();
            else
                ackString = AcDb.HostApplicationServices.Current.GetType().GetProperty("MachineRegistryProductRootKey").GetValue(AcDb.HostApplicationServices.Current, null).ToString();
            RegistryKey ack = hive.OpenSubKey(ackString, true);
            using (ack)
            {
                RegistryKey appk = ack.CreateSubKey("Applications");
                using (appk)
                {   // Already registered? Just return
                    // Create the our application's root key and its values 
                    RegistryKey rk = appk.CreateSubKey(name);
                    using (rk)
                    {
                        rk.SetValue("DESCRIPTION", name, RegistryValueKind.String);
                        rk.SetValue("LOADCTRLS", flags, RegistryValueKind.DWord);
                        rk.SetValue("LOADER", path, RegistryValueKind.String);
                        rk.SetValue("MANAGED", 1, RegistryValueKind.DWord);
                        // Create a subkey if there are any commands... 
                        if ((globCmds.Count == locCmds.Count) &&
                            globCmds.Count > 0)
                        {
                            RegistryKey ck = rk.CreateSubKey("Commands");
                            using (ck)
                            {
                                for (int i = 0; i < globCmds.Count; i++)
                                    ck.SetValue(globCmds[i], locCmds[i], RegistryValueKind.String);
                            }
                        }
                        // And the command groups, if there are any 
                        if (groups.Count > 0)
                        {
                            RegistryKey gk = rk.CreateSubKey("Groups");
                            using (gk)
                            {
                                foreach (string grpName in groups)
                                { gk.SetValue(grpName, grpName, RegistryValueKind.String); }
                            }
                        }
                    }
                }
            }
        }

        private static void RemoveDemandLoadingEntries(bool currentUser)
        {
            try
            {
                // Choose a Registry hive based on the function input
                RegistryKey hive = (currentUser ? Microsoft.Win32.Registry.CurrentUser : Microsoft.Win32.Registry.LocalMachine);
                // Open the main AutoCAD (vertical) and "Applications" keys
                String ackString = "";
                if (AcDb.HostApplicationServices.Current.GetType().GetProperty("RegistryProductRootKey") != null)
                    ackString = AcDb.HostApplicationServices.Current.GetType().GetProperty("RegistryProductRootKey").GetValue(AcDb.HostApplicationServices.Current, null).ToString();
                else
                    ackString = AcDb.HostApplicationServices.Current.GetType().GetProperty("MachineRegistryProductRootKey").GetValue(AcDb.HostApplicationServices.Current, null).ToString();
                RegistryKey ack = hive.OpenSubKey(ackString, true);
                //RegistryKey ack = hive.OpenSubKey(HostApplicationServices.Current.RegistryProductRootKey);
                using (ack)
                {
                    RegistryKey appk = ack.OpenSubKey("Applications", true);
                    using (appk)
                    { // Delete the key with the same name as this assembly
                        appk.DeleteSubKeyTree(ApplicationName);
                    }
                }
            }
            catch { }
        }
    }
}
