﻿using Autodesk.AutoCAD.DatabaseServices;
using Autodesk.AutoCAD.Geometry;
using System.Collections.Generic;
using System.Linq;


namespace Export3dSolids.Objects
{
    public class EntityBlocks: List<EntityBlock>
    {
        // Collection, die alle Id's der Entities im Modelspace sammelt
        ObjectIdCollection _EntityIdsInModelSpace { get; set; }

        //Constructor
        public EntityBlocks()
        {
            _EntityIdsInModelSpace = new ObjectIdCollection();
        }
        /// <summary>
        /// Export aller Entity-Objekte, egal welcher Verschachtelungstiefe in eine extra Zeichnung ohne Blöcke
        /// alle Entities befinden sich dann im ModelSpace
        /// </summary>
        /// <param name="_db">Current Database</param>
        /// <param name="_resultDwgName">Ergebnis-Datei-FullName</param>
        public void ToWblock(Database _db, string _resultDwgName)
        {
            using (Transaction _tr = _db.TransactionManager.StartTransaction())
            {
                while (this.Count > 0)
                {
                    ToParent(this[0], _tr);
                }
                _db.Wblock(_EntityIdsInModelSpace, Point3d.Origin).SaveAs(_resultDwgName, DwgVersion.Current);
            }
        }
        /// <summary>
        /// Transformiert alle Entities in den Parent-BlockTableRecord
        /// </summary>
        /// <param name="_current"></param>
        /// <param name="_tr"></param>
        private void ToParent(EntityBlock _current, Transaction _tr)
        {
            _current.BlockReferenceIds.All(_brId =>
            {
                using (BlockReference _br = (BlockReference)_tr.GetObject(_brId, OpenMode.ForRead))
                {
                    using (BlockTableRecord _oBtr = (BlockTableRecord)_tr.GetObject(_br.OwnerId, OpenMode.ForRead))
                    {
                        List<ObjectId> _EntityIdsInOwnerSpace = new List<ObjectId>();
                        // EntityObjekts kopieren
                        _current.EntityIds.All(_sId =>
                        {
                            using (Entity _ent = (Entity)_tr.GetObject(_sId, OpenMode.ForWrite, false, true))
                            {
                                using (Entity _newEnt = (Entity)_ent.Clone())
                                {
                                    // Entityobject aus der Blockdefinition in den Bereich der aktuellen Blockreferenz transformieren
                                    _newEnt.TransformBy(_br.BlockTransform);
                                    using (BlockTableRecord _ms = (BlockTableRecord)_tr.GetObject(_br.OwnerId, OpenMode.ForWrite))
                                    {
                                        // Entity in den Bereich der aktuellen Blockreferenz einfügen & ObjektId der Liste der neuen Solids hinzufügen
                                        _EntityIdsInOwnerSpace.Add(_ms.AppendEntity(_newEnt));
                                        _tr.AddNewlyCreatedDBObject(_newEnt, true);
                                    }
                                }
                            }
                            return true;
                        });
                        if (_oBtr.Name.ToUpper() != BlockTableRecord.ModelSpace.ToUpper())
                        { // Wenn Parent <> ModelSpace
                            // .. wird ein neuer EntityBlock erzeugt
                            EntityBlock _entBlock = new EntityBlock();
                            _entBlock.EntityIds = _EntityIdsInOwnerSpace;
                            ObjectIdCollection _refIds = _oBtr.GetBlockReferenceIds(true, true);
                            foreach (ObjectId _id in _refIds)
                            { _entBlock.BlockReferenceIds.Add(_id); }
                            this.Add(_entBlock);
                        }
                        else
                        { // Wenn Parent == ModelSpace
                            this.Remove(_current);
                            _EntityIdsInOwnerSpace.ForEach(_ => this._EntityIdsInModelSpace.Add(_));
                        }
                    }
                }
                return true;
            });
        }
    }
    
}
