﻿using Export3dSolids.Objects;
using Autodesk.AutoCAD.DatabaseServices;
using System;
using System.IO;


namespace Export3dSolids
{
    internal partial class Tool
    {
        public static void RunTool()
        {
            string _resultDwgName = @"c:\Temp\result.dwg";
            string _layername = "0";
            #region -> Zieldatei - Handling 
            bool _canSave = true;
            if (File.Exists(_resultDwgName))
            {
                string _bakFileName = _resultDwgName.Replace(".dwg", ".bak");
                if (File.Exists(_bakFileName))
                {
                    try { File.Delete(_bakFileName); }
                    catch
                    {
                        _canSave = false;
                    }
                }
                if (!_canSave)
                {
                    try { File.Delete(_resultDwgName); _canSave = true; }
                    catch
                    { }
                }
                if (_canSave)
                { File.Move(_resultDwgName, _bakFileName); }
            }
            #endregion <- Zieldatei - Handling 
            #region -> Wenn Zieldatei nicht geschrieben werden kann, weil dadurch existierende Datei überschrieben würde ...
            if (!_canSave)
            {
                Autodesk.AutoCAD.ApplicationServices.Application.ShowAlertDialog("Ergebnisdatei kann nicht geschrieben werden!\n\nABBRUCH!");
                return;
            }
            #endregion <- 

            Database _db = HostApplicationServices.WorkingDatabase;
            // Liste der Ids der kopierten Solids
            ObjectIdCollection _SolidIdsInModelSpace = new ObjectIdCollection();
            EntityBlocks _BlocksWithSolids = new EntityBlocks();
            using (Transaction _tr = _db.TransactionManager.StartTransaction())
            {
                using (BlockTable _bt = (BlockTable)_tr.GetObject(_db.BlockTableId,  OpenMode.ForRead))
                { // Alle Blöcke durchlaufen
                    foreach (ObjectId _oid in _bt)
                    {
                        using (BlockTableRecord _btr = (BlockTableRecord)_tr.GetObject(_oid, OpenMode.ForRead))
                        {
                            // alle Blockreferenzen ermitteln
                            ObjectIdCollection _refIds = _btr.GetBlockReferenceIds(true, true);
                            EntityBlock _solBlock = null;
                            foreach (ObjectId _eid in _btr)
                            { // Alle Elemente eines Blocks durchlaufen
                                using (Entity _ent = (Entity)_tr.GetObject(_eid, OpenMode.ForRead))
                                {
                                    if (_ent.Layer == _layername && typeof(Solid3d).IsAssignableFrom(_ent.GetType()))
                                    {
                                        // Wenn Element = Solid3d auf Layer
                                        if (_solBlock == null)
                                        {
                                            _solBlock = new EntityBlock();
                                            foreach (ObjectId _id in _refIds)
                                            { _solBlock.BlockReferenceIds.Add(_id); }
                                            _BlocksWithSolids.Add(_solBlock);
                                        }
                                        _solBlock.EntityIds.Add(_eid);
                                    }
                                }
                            }
                        }
                    }
                }
            }
            // Solids exportieren.
            _BlocksWithSolids.ToWblock(_db, _resultDwgName);
        }

    }
}